@@ -35,8 +35,8 @@
 #define COMMON_OPTIONS_SINGLE \
   AddOption("o","output","output file");
 
-#define COMMON_OPTIONS_CLOBBER	\
-  AddSwitch("","no-clobber","do not overwrite existing files");
+#define COMMON_OPTIONS_ENABLE_OVERWRITE	\
+  AddSwitch("","enable-overwrite","overwrite existing files");
 
 #define COMMON_OPTIONS_MULTIPLE \
   AddOption("t","target-directory","set target directory");		\
@@ -89,7 +89,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     for(int i = 0; i < xargc; i++) {
       wxString arg(xargv[i]);
       if( arg != action && arg != "-" &&  arg.substr(0,1) != "@" )
-	argv[n++] = wxStrdup(arg.wc_str());
+	argv[n++] = wxStrdup(xargv[i]);
     }
     wxASSERT(n == argc);
 
@@ -123,6 +123,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
 		 " votable\tConversions of VOTables\n"
 		 " fits\t\tOperations on FITS files\n"
 		 " cross\t\tCross-match on FITS tables\n"
+		 " sesame\t\tAstronomical name resolver\n"
 		 );
 
     AddSwitch("","version","print version and license");
@@ -193,7 +194,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     AddOption("xbitmask","","mask frame as ZERO, MEAN(default), MEDIAN");
     AddOption("box","","box size for MEDIAN bitmask",wxCMD_LINE_VAL_NUMBER);
     AddSwitch("","normalise-flat","normalise flat-field prior to use");
-    COMMON_OPTIONS_CLOBBER
+    COMMON_OPTIONS_ENABLE_OVERWRITE
     COMMON_OPTIONS_MULTIPLE
     COMMON_OPTIONS_BITPIX
     AddParam("file(s)",wxCMD_LINE_VAL_STRING,
@@ -271,6 +272,13 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     AddParam("file",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
   }
 
+  else if( action == "sesame" ) {
+
+    AddUsageText("Usage: munipack "+action+" [options] object");
+    AddUsageText("* A name resolver of objects by Virtual Observatory.");
+    AddParam("object",wxCMD_LINE_VAL_STRING);
+  }
+
   else if( action == "astrometry" ) {
 
     AddUsageText("Usage: munipack "+action+" [options] file(s)[,results(s)]");
@@ -361,6 +369,12 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
 
     AddOption("","apertures","a1,a2,... define aperture radii in pixels");
     AddOption("","ring","ri,ro inner and outer sky ring radius in pixels");
+    AddSwitch("","enable-ellipticity","enable elliptic apertures");
+    AddOption("","eccentricity","of apertures (by default: circle)",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","inclination","inclination of major semiaxis, degrees",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","coo","coordinates of objects");
     COMMON_OPTIONS_MULTIPLE
     AddParam("file(s)",wxCMD_LINE_VAL_STRING,
 	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
@@ -429,7 +443,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
   //   //    AddSwitch(wxEmptyString,"mag","output in instrumental magnitudes");
   //   //    AddSwitch(wxEmptyString,"full","full list");
   //   //    AddSwitch(wxEmptyString,"pos","position list");
-  //   AddOption(wxEmptyString,"longitude","geographical longitude of station (-east)[deg]",wxCMD_LINE_VAL_DOUBLE);
+  //   AddOption(wxEmptyString,"longitude","geographical longitude of station (+east)[deg]",wxCMD_LINE_VAL_DOUBLE);
   //   AddOption(wxEmptyString,"latitude","geographical latitude of station (+north)[deg]",wxCMD_LINE_VAL_DOUBLE);
   //   AddOption(wxEmptyString,"ra","Right Ascension [deg]",wxCMD_LINE_VAL_DOUBLE);
   //   AddOption(wxEmptyString,"dec","Declination [deg]",wxCMD_LINE_VAL_DOUBLE);
@@ -558,7 +572,6 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     AddSwitch(wxEmptyString,"list","show available photometry systems");
     AddOption(wxEmptyString,"tol","search radius [deg]",wxCMD_LINE_VAL_DOUBLE);
     AddOption("","saper","select aperture number",wxCMD_LINE_VAL_NUMBER);
-    COMMON_OPTIONS_CLOBBER
     COMMON_OPTIONS_MULTIPLE
     AddParam("file(s)[,results(s)]",wxCMD_LINE_VAL_STRING,
   	     wxCMD_LINE_PARAM_MULTIPLE|wxCMD_LINE_PARAM_OPTIONAL);
@@ -599,6 +612,10 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
 	      wxCMD_LINE_VAL_DOUBLE);
     AddOption("","airy","radius of Airy spot [pix]",wxCMD_LINE_VAL_DOUBLE);
     AddOption("","beta","Moffat exponent",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","eccentricity","of ellipsis (by default: circle)",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","inclination","inclination of major semiaxis, degrees",
+	      wxCMD_LINE_VAL_DOUBLE);
     AddOption("","maglim","magnitude limit",wxCMD_LINE_VAL_NUMBER);
     AddOption("","lc-model","light curve model: Cepheid,Algol,wave(default)");
     AddOption("","lc-table","light curve model in the table");
@@ -627,7 +644,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     AddOption("","qeff","quantum efficiency",wxCMD_LINE_VAL_DOUBLE);
     AddSwitch("","atmosphere","apply atmosphere modelling: extinction+seeing");
     AddOption("","extk","extinction coefficient",wxCMD_LINE_VAL_DOUBLE);
-    AddOption("","long","geographic longitude of station (-east)[deg]",
+    AddOption("","long","geographic longitude of station (+east)[deg]",
 	      wxCMD_LINE_VAL_DOUBLE);
     AddOption("","lat","geographic latitude of station (+north)[deg]",
 	      wxCMD_LINE_VAL_DOUBLE);
@@ -655,6 +672,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
     AddOption("","col-pm-ra","Proper motion in Right Ascension column");
     AddOption("","col-pm-dec","Proper motion in Declination column");
     AddOption("","col-mag","Magnitude-like column");
+    AddSwitch("","disable-noise","noise free frames");
     AddOption("","photsys","photometry system");
     AddOption("","phsystab","photometry systems definition table");
     AddOption("","mask","mask for output files (artificial_???.fits)");
