@@ -35,17 +35,16 @@
 using namespace std;
 
 
-
-
 // -- MuniDisplayCanvas
 
 MuniDisplayCanvas::MuniDisplayCanvas(wxWindow *w, MuniConfig *c):
   wxScrolledCanvas(w,wxID_ANY),render(0),canvasid(0),
   topwin(GetGrandParent()),config(c),handler(0),
   xoff(0), yoff(0), value_type(UNIT_COUNT), coo_type(COO_PIXEL),
-  scaleicon(false), shrink(1),zoom(1),palauto(false),
+  scaleicon(false), shrink(1),zoom(1),
   zooming(false), shrinking(false),tunning(false), completed(false),
-  dragging(false),rendering(false), astrometry(false)
+  dragging(false),invoking(false), rendering(false), finished(false),
+  astrometry(false)
 {
   wxLogDebug("MuniDisplayCanvas constructor..");
 
@@ -60,6 +59,7 @@ MuniDisplayCanvas::MuniDisplayCanvas(wxWindow *w, MuniConfig *c):
   Bind(wxEVT_IDLE,&MuniDisplayCanvas::OnIdle,this);
   Bind(EVT_RENDER,&MuniDisplayCanvas::OnRenderFinish,this);
   Bind(EVT_RENDER,&MuniDisplayCanvas::OnSubRender,this,ID_SUBRENDER);
+  Bind(EVT_DRAW,&MuniDisplayCanvas::OnDraw,this);
   Bind(wxEVT_KEY_DOWN,&MuniDisplayCanvas::OnKeyDown,this);
   Bind(wxEVT_RIGHT_DOWN,&MuniDisplayCanvas::OnMenu,this);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnClipValue,this,
@@ -70,6 +70,15 @@ MuniDisplayCanvas::MuniDisplayCanvas(wxWindow *w, MuniConfig *c):
        ID_VALTYPE);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnCooType,this,
        ID_COOTYPE);
+
+  // SetHandler clone
+  Bind(wxEVT_MOTION,&MuniDisplayCanvas::OnMouseMotion,this);
+  Bind(wxEVT_ENTER_WINDOW,&MuniDisplayCanvas::OnMouseEnter,this);
+  Bind(wxEVT_LEAVE_WINDOW,&MuniDisplayCanvas::OnMouseLeave,this);
+  Bind(wxEVT_LEFT_UP,&MuniDisplayCanvas::OnLeftUp,this);
+  Bind(wxEVT_LEFT_DOWN,&MuniDisplayCanvas::OnClick,this);
+  Bind(EVT_TUNE,&MuniDisplayCanvas::OnTuneFine,this);
+
 }
 
 MuniDisplayCanvas::~MuniDisplayCanvas()
@@ -80,10 +89,6 @@ MuniDisplayCanvas::~MuniDisplayCanvas()
     delete *i;
   */
 
-  if( ! palauto ) {
-    config->display_pal = pal.GetPalette();
-    config->display_palinv = pal.GetInverse();
-  }
 }
 
 void MuniDisplayCanvas::OnClose(wxCloseEvent& event)
@@ -91,7 +96,8 @@ void MuniDisplayCanvas::OnClose(wxCloseEvent& event)
   //  wxLogDebug("MuniDisplayCanvas::OnClose");
 
   if( render ) {
-    render->Delete();
+    StopRendering();
+    //    render->Delete();
     event.Veto();
     wxQueueEvent(this,new wxCloseEvent());
   }
@@ -103,18 +109,26 @@ void MuniDisplayCanvas::OnIdle(wxIdleEvent& event)
 {
   //  wxLogDebug("MuniDisplayCanvas::OnIdle");
 
-  // Rendering is started by Idle event FOLLOWING all (resize) events
-  // The right canvas size is prepared for drawing.
+  // Rendering is started by Idle event FOLLOWING all resize(!) events
+  // The canvas with a right size is prepared for drawing.
+
+  if( invoking ) {
 
-  if( rendering ) {
     if( render ) {
       StopRendering();
-      event.RequestMore();
-    }
-    else {
-      rendering = false;
-      StartRendering();
+      //      event.RequestMore();
+      //      return;
     }
+
+    invoking = false;
+    finished = false;
+    ::wxBeginBusyCursor();
+    StartRendering();
+  }
+
+  if( finished ) {
+    ::wxEndBusyCursor();
+    rendering = false;
   }
 
 }
@@ -126,6 +140,8 @@ void MuniDisplayCanvas::OnSize(wxSizeEvent& event)
 
 void MuniDisplayCanvas::OnPaint(wxPaintEvent& event)
 {
+  //  wxLogDebug("MuniDisplayCanvas::OnPaint");
+
   wxPaintDC dc(this);
   DoPrepareDC(dc);
 
@@ -203,25 +219,9 @@ void MuniDisplayCanvas::OnPaint(wxPaintEvent& event)
 
 }
 
-
-void MuniDisplayCanvas::SetHandler(wxEvtHandler *h)
-{
-  handler = h;
-
-  if( handler ) {
-    Bind(wxEVT_MOTION,&MuniDisplayCanvas::OnMouseMotion,this);
-    //    Bind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
-    Bind(wxEVT_LEFT_UP,&MuniDisplayCanvas::OnLeftUp,this);
-  }
-  else {
-    Unbind(wxEVT_MOTION,&MuniDisplayCanvas::OnMouseMotion,this);
-    //    Unbind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
-    Unbind(wxEVT_LEFT_UP,&MuniDisplayCanvas::OnLeftUp,this);
-  }
-}
-
-void MuniDisplayCanvas::SetHdu(const FitsArray& a, const wxImage& i, bool pauto)
+void MuniDisplayCanvas::SetHdu(const FitsArray& a, const wxImage& i)
 {
+  wxLogDebug("MuniDisplayCanvas::SetHdu EXTNAME="+a.GetExtname()+" %p",this);
   array = a;
   icon = i;
   if( icon.HasAlpha() )
@@ -231,43 +231,31 @@ void MuniDisplayCanvas::SetHdu(const FitsArray& a, const wxImage& i, bool pauto)
   fitsimage = FitsImage(array);
   wxASSERT(fitsimage.IsOk());
 
-  FitsArray stat(array.IsColor() ? array.Plane(1) : array);
+  FitsArray stat(array.IsColour() ? array.Plane(1) : array);
   tone.Setup(stat.Npixels(),stat.PixelData());
 
-  palauto = pauto;
-  if( palauto )
-    SelectAutoPal();
-
   // Init canvas
+  shrink = 1;
   InitCanvas();
-  rendering = true;
+  //  rendering = true;
   shrinking = true;
   tunning = true;
+}
 
-  // setup zoom
-  MuniFullTuneEvent t(EVT_FULLTUNE);
-  t.tone = tone;
-  t.itt = itt;
-  t.pal = pal;
-  t.colour = color;
-  wxQueueEvent(handler,t.Clone());
-
-  Refresh();
+void MuniDisplayCanvas::SetInitShrink(int s)
+{
+  shrink = s;
+  InitCanvas();
 }
 
 void MuniDisplayCanvas::InitCanvas()
 {
   canvasid = 007;
-  shrink = GetFitShrink();
+  //  shrink = 1;
   zoom = 1;
   canvas = icon.Scale(fitsimage.GetWidth()/shrink,fitsimage.GetHeight()/shrink);
   SetVirtualSize(canvas.GetWidth(),canvas.GetHeight());
   UpdateScroll();
-
-  MuniTuneEvent ev(EVT_TUNE,ID_ZOOM);
-  ev.n = shrink;
-  ev.shrink = true;
-  wxQueueEvent(topwin,ev.Clone());
 }
 
 void MuniDisplayCanvas::UpdateCanvas(int shrink, int zoom)
@@ -300,7 +288,8 @@ void MuniDisplayCanvas::Reset()
   pal.SetPalette(config->display_pal);
 
   InitCanvas();
-  rendering = true;
+  //  rendering = true;
+  invoking = true;
   shrinking = true;
   tunning = true;
 }
@@ -309,7 +298,7 @@ void MuniDisplayCanvas::StartRendering()
 {
   wxASSERT(render == 0);
 
-  ::wxBeginBusyCursor();
+  //  ::wxBeginBusyCursor();
 
   if( shrinking ) {
     completed = false;
@@ -356,13 +345,14 @@ void MuniDisplayCanvas::OnSubRender(MuniRenderEvent& event)
 
 void MuniDisplayCanvas::OnRenderFinish(MuniRenderEvent& event)
 {
-  wxLogDebug("MuniDisplayCanvas::OnRenderFinish %d ",
-	     event.GetId()==ID_RENDER_SHRINK);
+  wxLogDebug("MuniDisplayCanvas::OnRenderFinish %d %d %p",event.id,event.GetId(),this);
+
+  finished = true;
 
   if( event.image.IsOk() && event.GetId() == ID_RENDER_SHRINK &&
       event.completed ) {
-    completed = event.completed;
     scaled = event.image;
+    completed = event.completed;
   }
 
   if( event.GetId() == ID_RENDER_SHRINK && scaled.IsOk() ) {
@@ -370,7 +360,14 @@ void MuniDisplayCanvas::OnRenderFinish(MuniRenderEvent& event)
     tunning = true;
   }
 
-  ::wxEndBusyCursor();
+  if( event.completed )
+    wxQueueEvent(GetParent(),event.Clone());
+
+}
+
+void MuniDisplayCanvas::InvokeRendering()
+{
+  invoking = true;
 }
 
 void MuniDisplayCanvas::StopRendering()
@@ -391,8 +388,26 @@ void MuniDisplayCanvas::StopRendering()
   }
 
   rendering = false;
+  finished = true;
 }
 
+void MuniDisplayCanvas::OnDraw(MuniDrawEvent& event)
+{
+  //  wxLogDebug("MuniDisplayCanvas::OnDraw");
+  MuniLayer layer(event.layer);
+
+  // remove all layers identified by MuniLayer::id
+  list<MuniLayer>::iterator i = layers.begin();
+  while( i != layers.end() ) {
+    if( i->GetId() == event.layer.GetId() )
+      i = layers.erase(i);
+    else
+      ++i;
+  }
+  layers.push_back(event.layer);
+
+  Refresh();
+}
 
 void MuniDisplayCanvas::ConvertCoo(double u, double v, double& x, double& y)
 {
@@ -511,6 +526,53 @@ void MuniDisplayCanvas::DrawLayer(wxPaintDC& dc, const MuniLayer& layer)
 	gc->DrawEllipse(x,y,d,d);
       }
 
+      MuniDrawEllipse *ellipse = dynamic_cast<MuniDrawEllipse *>(*i);
+      if( ellipse ) {
+	double a = ellipse->a;
+	double e = ellipse->e;
+	double b = a*sqrt(1 - e*e);
+	double x,y;
+	//	wxGraphicsMatrix m = gc->GetTransform();
+	ConvertCoo(ellipse->x,ellipse->y,x,y);
+	gc->Translate(x,y);
+	gc->Rotate(ellipse->i / 57.3);
+	gc->DrawEllipse(-a,-b,2*a,2*b);
+	gc->Rotate(-ellipse->i / 57.3);
+	gc->Translate(-x,-y);
+      }
+
+      MuniDrawRing *ring = dynamic_cast<MuniDrawRing *>(*i);
+      if( ring ) {
+	double rin = ring->rin;
+	double rout = ring->rout;
+	double x,y;
+	ConvertCoo(ring->x,ring->y,x,y);
+	wxGraphicsPath gpath = gc->CreatePath();
+	gpath.AddEllipse(x-rin,y-rin,2*rin,2*rin);
+	gpath.AddEllipse(x-rout,y-rout,2*rout,2*rout);
+	gc->FillPath(gpath);
+
+	/*
+	double e = ring->e;
+	double q = sqrt(1 - e*e);
+	double x,y,a,b;
+	ConvertCoo(ring->x,ring->y,x,y);
+	gc->Translate(x,y);
+	gc->Rotate(-ring->i / 57.3);
+	wxGraphicsPath gpath = gc->CreatePath();
+	a = rin;
+	b = q*rin;
+	gpath.AddEllipse(-a,-b,2*a,2*b);
+	a = rout;
+	b = q*rout;
+	gpath.AddEllipse(-a,-b,2*a,2*b);
+	gc->FillPath(gpath);
+	gc->Rotate(ring->i / 57.3);
+	gc->Translate(-x,-y);
+	*/
+      }
+
+
       MuniDrawLine *line = dynamic_cast<MuniDrawLine *>(*i);
       if( line ) {
 	wxPoint2DDouble points[2];
@@ -716,13 +778,15 @@ void MuniDisplayCanvas::OnMouseMotion(wxMouseEvent& event)
 
   if( event.Moving() ) {
 
-    MuniSlewEvent evt(xEVT_SLEW);
+    MuniSlewEvent evt(EVT_SLEW);
     crosshair = GetCartesianPosition(event.GetLogicalPosition(dc));
     evt.x = crosshair.x;
     evt.y = crosshair.y;
 
-    wxASSERT(handler);
-    wxQueueEvent(handler,evt.Clone());
+    wxRect rect(0,0,fitsimage.GetWidth(),fitsimage.GetHeight());
+    evt.inside = rect.Contains(crosshair);
+
+    wxQueueEvent(GetParent(),evt.Clone());
   }
   else if( event.Dragging() ) {
 
@@ -751,6 +815,20 @@ void MuniDisplayCanvas::OnMouseMotion(wxMouseEvent& event)
 
 }
 
+void MuniDisplayCanvas::OnMouseEnter(wxMouseEvent& event)
+{
+  MuniSlewEvent evt(EVT_SLEW);
+  evt.entering = true;
+  wxQueueEvent(GetParent(),evt.Clone());
+}
+
+ void MuniDisplayCanvas::OnMouseLeave(wxMouseEvent& event)
+{
+  MuniSlewEvent evt(EVT_SLEW);
+  evt.leaving = true;
+  wxQueueEvent(GetParent(),evt.Clone());
+}
+
 void MuniDisplayCanvas::OnMouseWheel(wxMouseEvent& event)
 {
   /*
@@ -759,6 +837,16 @@ void MuniDisplayCanvas::OnMouseWheel(wxMouseEvent& event)
   */
 }
 
+void MuniDisplayCanvas::OnClick(wxMouseEvent& event)
+{
+  wxClientDC dc(this);
+  wxPoint p = GetCartesianPosition(event.GetLogicalPosition(dc));
+  MuniClickEvent e(EVT_CLICK);
+  e.x = p.x;
+  e.y = p.y;
+  wxQueueEvent(GetParent(),e.Clone());
+}
+
 void MuniDisplayCanvas::OnLeftUp(wxMouseEvent& event)
 {
   if( dragging ) {
@@ -776,12 +864,6 @@ void MuniDisplayCanvas::OnMenu(wxMouseEvent& event)
 
   if( twin->IsFullScreen() ) {
     popup.Append(ID_MENU_FULLSCREEN,"Leave Fullscreen");
-    popup.AppendSeparator();
-    popup.Append(wxID_ZOOM_100,"Normal size\tCtrl+0");
-    popup.Append(wxID_ZOOM_IN,"Zoom\tCtrl++");
-    popup.Append(wxID_ZOOM_OUT,"Shrink\tCtrl+-");
-    popup.Append(wxID_ZOOM_FIT,"Best fit\tCtrl+*");
-    popup.Append(ID_CYCLE_PAL,"Cycle Palette\tCtrl+P");
   }
   else /*if( ! twin->IsFullScreen() )*/ {
     popup.Append(ID_CLIP_VALUE,"Copy value");
@@ -799,7 +881,7 @@ void MuniDisplayCanvas::OnClipValue(wxCommandEvent& event)
   value.SetType(value_type);
 
   wxString val;
-  if( array.IsColor() ) {
+  if( array.IsColour() ) {
     val =       "X = " + value.Get_str(p.x,p.y,2);
     val = val + " Y = " + value.Get_str(p.x,p.y,1);
     val = val + " Z = " + value.Get_str(p.x,p.y,0);
@@ -867,10 +949,6 @@ void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
       ;
     else {
       switch(keycode) {
-      case WXK_NUMPAD_ADD:      id = wxID_ZOOM_IN;  break;
-      case WXK_NUMPAD_SUBTRACT: id = wxID_ZOOM_OUT; break;
-      case WXK_NUMPAD0:         id = wxID_ZOOM_100; break;
-      case WXK_NUMPAD_MULTIPLY: id = wxID_ZOOM_FIT; break;
       case WXK_LEFT:            id = wxID_BACKWARD; break;
       case WXK_RIGHT:           id = wxID_FORWARD;  break;
       }
@@ -902,53 +980,40 @@ void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
 
 void MuniDisplayCanvas::UpdateTune()
 {
-  rendering = true;
+  //rendering = true;
+  invoking = true;
   tunning = true;
   UpdateCanvas();
-
-  // inform zoom window on panel
-  MuniFullTuneEvent t(EVT_FULLTUNE);
-  t.tone = tone;
-  t.itt = itt;
-  t.pal = pal;
-  t.colour = color;
-  wxQueueEvent(handler,t.Clone());
 }
 
 void MuniDisplayCanvas::UpdateScale()
 {
-  rendering = true;
+  //  rendering = true;
+  invoking = true;
   //  scaling = false;
   shrinking = true;
   tunning = true;
   UpdateCanvas();
-
-  // inform zoom window on panel
-  MuniFullTuneEvent t(EVT_FULLTUNE);
-  t.tone = tone;
-  t.itt = itt;
-  t.pal = pal;
-  t.colour = color;
-  wxQueueEvent(handler,t.Clone());
 }
 
-void MuniDisplayCanvas::UpdateScale(bool down, int s)
+void MuniDisplayCanvas::UpdateScale(int s)
 {
   shrinking = true;
   int oldshrink = shrink;
 
-  if( down ) {
+  if( s > 0 ) {
     shrink = s;
     zoom = 1;
-    wxLogDebug("MuniDisplayCanvas::OnTuneFine shrink: 1/%d",shrink);
+    wxLogDebug("MuniDisplayCanvas::OnTuneFine shrink: 1:%d",shrink);
   }
   else {
     shrink = 1;
-    zoom = s;
-    wxLogDebug("MuniDisplayCanvas::OnTuneFine zoom: %d x",zoom);
+    zoom = abs(s);
+    wxLogDebug("MuniDisplayCanvas::OnTuneFine zoom: %d:1",zoom);
   }
 
-  rendering = oldshrink != 1;
+  //  rendering = oldshrink != 1;
+  invoking = oldshrink != 1;
   wxLogDebug("MuniDisplayCanvas::UpdateScale: %d %d %d %d %d",
 	     oldshrink,zoom,completed,rendering,zooming);
 
@@ -971,6 +1036,7 @@ void MuniDisplayCanvas::UpdateScroll()
     dw = zoom;
   if( h > hc )
     dh = zoom;
+
   if( dw > 0 || dh > 0 )
     SetScrollRate(dw,dh);
 
@@ -1025,7 +1091,7 @@ void MuniDisplayCanvas::OnTuneFine(MuniTuneEvent& e)
 
   // shrink or zoom
   if( e.GetId() == ID_ZOOM_SCALE ) {
-    UpdateScale(e.shrink,e.n);
+    UpdateScale(e.n);
     return;
   }
 
@@ -1109,60 +1175,22 @@ void MuniDisplayCanvas::RemoveLayers(int id)
   Refresh();
 }
 
-int MuniDisplayCanvas::GetFitShrink() const
+
+wxImage MuniDisplayCanvas::GetImage() const
 {
-  if( ! array.IsOk() ) return 1;
+  /*
+    Returns non-scaled or non-shrink size of frame.
+    That's a placeholder, the proper rendering is necessary.
 
-  const int nratios = 6;
-  const int ratios[6] = { 1, 2, 3, 4, 6, 8};
+  */
 
-  wxSize client = topwin->GetClientSize();
-
-  int n = ratios[nratios-1];
-  int width = array.Width();
-  int height = array.Height();
-  for(int l = 0; l < nratios; l++) {
-    width = array.Width() / ratios[l];
-    height = array.Height() / ratios[l];
-    if( (width < client.GetWidth() && height < client.GetHeight()) ) {
-      n = ratios[l];
-      break;
-    }
-  }
-  return n;
-}
-
-FitsZoom MuniDisplayCanvas::GetZoom() const
-{
-  if( shrink > 0 )
-    return FitsZoom(shrink);
-  else if( zoom > 0 )
-    return FitsZoom(zoom,ZOOM_ZOOM);
+  if( shrink == 1 )
+    return canvas;
   else
-    return FitsZoom(1);
+    return canvas.Scale(array.GetWidth(),array.GetHeight());
 }
 
-void MuniDisplayCanvas::OnMenuPalauto(wxCommandEvent& event)
-{
-  palauto = event.IsChecked();
-  if( palauto )
-    SelectAutoPal();
-  else
-    pal.SetPalette(config->display_pal);
-  UpdateTune();
-}
 
-void MuniDisplayCanvas::SelectAutoPal()
-{
-  //  wxLogDebug("MuniDisplayCanvas::SelectAutoPal");
-  wxString filter(array.GetKey(config->fits_key_filter));
-  if( filter.Find("R") != wxNOT_FOUND )
-    pal.SetPalette(PAL_HEAT);
-  else if( filter.Find("V") != wxNOT_FOUND|| filter.Find("G") != wxNOT_FOUND)
-    pal.SetPalette(PAL_SPRING);
-  else if( filter.Find("B") != wxNOT_FOUND )
-    pal.SetPalette(PAL_COOL);
-}
 
 /*
 void MuniDisplayCanvas::OverlayBitmap(const vector<MuniDrawBase *>& drawobj)
