@@ -22,6 +22,7 @@
 */
 
 #include "fits.h"
+#include <string>
 #include <cstdio>
 #include <fitsio.h>
 #include <cmath>
@@ -45,7 +46,7 @@ using namespace std;
 // we know the types of images:
 // * 1d (spectrum)
 // * 2d grayscale
-// * 2d color (3d, unique sizes, header with CSPACE keyword)
+// * 3d colour (3d, unique sizes, header with CSPACE keyword)
 // * 3d (3d spectrum) ?
 // * multilayer (more layers, arbitrary sizes)
 //
@@ -86,7 +87,7 @@ bool FitsHeader::ParseRecord(const wxString& record, wxString& key,
 			     wxString& value, wxString& comment)
 {
   int status = 0;
-  const char *card = record.c_str();
+  const char *card = static_cast<const char *>(record);
   char keyname[FLEN_CARD],cval[FLEN_CARD],com[FLEN_CARD];
   int keylen;
 
@@ -296,7 +297,7 @@ bool FitsHdu::IsOk() const
   return ! header.IsEmpty();
 }
 
-bool FitsHdu::IsColor() const
+bool FitsHdu::IsColour() const
 {
   return Naxis() == 3;
 }
@@ -495,8 +496,8 @@ int FitsArray::Flavour() const
   FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
   wxASSERT(data && data->array && data->naxes);
 
-  if( IsColor() )
-    return HDU_IMAGE_COLOR;
+  if( IsColour() )
+    return HDU_IMAGE_COLOUR;
 
   switch(data->naxis) {
   case 1: return HDU_IMAGE_LINE;
@@ -510,9 +511,9 @@ wxString FitsArray::Flavour_str() const
 {
   switch (Flavour()) {
   case HDU_IMAGE_LINE: return "Line";
-  case HDU_IMAGE_FRAME:return "Frame";
+  case HDU_IMAGE_FRAME:return "Gray";
   case HDU_IMAGE_CUBE: return "Cube";
-  case HDU_IMAGE_COLOR: return "Color frame";
+  case HDU_IMAGE_COLOUR: return "Colour";
   }
   wxLogDebug("FitsArray::Type_str(): Image type unknown");
   return wxEmptyString;
@@ -1451,6 +1452,16 @@ FitsHdu FitsFile::Hdu(size_t n) const
   return hdu[n];
 }
 
+FitsHdu FitsFile::FindHdu(const wxString& extname) const
+{
+  for(size_t n = 0; n < hdu.size(); n++) {
+    if( hdu[n].GetExtname() == extname )
+      return hdu[n];
+  }
+  return FitsHdu();
+}
+
+
 void FitsFile::Recognize()
 {
   type = FITS_UNKNOWN;
@@ -1461,9 +1472,10 @@ void FitsFile::Recognize()
     // simple, grayscale image
     type = FITS_GRAY;
 
-    // color image
-    if( hdu[0].IsColor() )
-      type = FITS_COLOR;
+    // colour image
+    FitsArray a(hdu[0]);
+    if( a.Naxis() == 3 )
+      type = FITS_COLOUR;
 
   }
 
@@ -1483,8 +1495,8 @@ wxString FitsFile::Type_str() const
 {
   switch (type) {
   case FITS_GRAY: return  "Gray image";
-  case FITS_COLOR: return "Color image";
-  case FITS_MULTI:return  "Multi layer";
+  case FITS_COLOUR: return "Colour image";
+  case FITS_MULTI:return  "Multi-extension";
   default: return wxEmptyString;
   }
 }
@@ -1499,9 +1511,18 @@ bool FitsFile::HasImage() const
   return false;
 }
 
+bool FitsFile::HasFind() const
+{
+  // locate FIND extension
+  for(size_t k = 0; k < HduCount(); k++)
+    if( hdu[k].GetExtname() == FINDEXTNAME )
+      return true;
+  return false;
+}
+
 bool FitsFile::HasPhotometry() const
 {
-  // locate extension
+  // locate APHOT extension
   for(size_t k = 0; k < HduCount(); k++)
     if( hdu[k].GetExtname() == APEREXTNAME )
       return true;
@@ -1510,7 +1531,7 @@ bool FitsFile::HasPhotometry() const
 
 bool FitsFile::HasPhcal() const
 {
-  // locate extension
+  // locate photometry extension
   for(size_t k = 0; k < HduCount(); k++)
     if( hdu[k].GetExtname() == PHCALNAME )
       return true;
@@ -1537,12 +1558,18 @@ wxString FitsFile::GetFullName() const
 wxString FitsFile::GetPath() const
 {
   wxFileName name(filename);
+  if( name.GetPath().IsEmpty() )
+    name.AssignCwd();
   return name.GetPath();
 }
 
 wxString FitsFile::GetFullPath() const
 {
   wxFileName name(filename);
+  if( name.GetPath().IsEmpty() ) {
+    name.AssignCwd();
+    name.SetFullName(filename);
+  }
   return name.GetFullPath();
 }
 
@@ -1778,6 +1805,31 @@ bool FitsCopyFile(const wxString& in, const wxString& out)
   return result && stat == 0;
 }
 
+bool FitsCopyHdu(const wxString& in, const wxString& out, const wxString& hdu)
+{
+  fitsfile *fin,*fout;
+  int stat = 0;
+  int hdutype;
+  bool result = false;
+  const char *extname = static_cast<const char *>(hdu);
+
+  fits_open_file(&fin, in.fn_str(), READONLY, &stat);
+  fits_open_file(&fout, out.fn_str(), READWRITE, &stat);
+  fits_movnam_hdu(fin,BINARY_TBL,(char*)extname,0,&stat);
+  if( fits_movnam_hdu(fout,BINARY_TBL,(char*)extname,0,&stat) == 0 )
+    fits_delete_hdu(fout,&hdutype,&stat);
+  if( stat == BAD_HDU_NUM )
+    stat = 0;
+  result = fits_copy_hdu(fin,fout,0,&stat) == 0;
+  fits_close_file(fout,&stat);
+  fits_close_file(fin,&stat);
+
+  fits_report_error(stderr,stat);
+
+  return result && stat == 0;
+}
+
+
 wxArrayString FitsColumns(const wxString& name)
 {
   fitsfile *f;
