@@ -3,7 +3,7 @@
   XMunipack
 
 
-  Copyright © 2009-2015, 2017-8 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2015, 2017-20 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -24,9 +24,8 @@
 
 #ifndef _XMUNIPACK_H_
 #define _XMUNIPACK_H_
-#endif
 
-#include "version.h"
+#include "../config.h"
 #include "config.h"
 #include "fits.h"
 #include "enum.h"
@@ -35,63 +34,45 @@
 #include "mprocess.h"
 #include "mconfig.h"
 #include "vocatconf.h"
-
-#ifndef _XMUNIPACK_ENUM_H_
+#include "aphot.h"
 #include "enum.h"
-#endif
-
-#ifndef _FITS_H_
 #include "fits.h"
-#endif
-
-#ifndef _XMUNIPACK_ICON_H_
 #include "icon.h"
-#endif
-
 #include <wx/wx.h>
 #include <wx/string.h>
 #include <wx/arrstr.h>
-#include <wx/list.h>
 #include <wx/imaglist.h>
 #include <wx/listctrl.h>
 #include <wx/srchctrl.h>
-#include <wx/html/htmlwin.h>
 #include <wx/dataview.h>
 #include <wx/listimpl.cpp>
 #include <wx/propdlg.h>
-#include <wx/tglbtn.h>
 #include <wx/process.h>
 #include <wx/thread.h>
 #include <wx/txtstrm.h>
 #include <wx/timer.h>
 #include <wx/filepicker.h>
 #include <wx/spinctrl.h>
-#include <wx/minifram.h>
 #include <wx/splitter.h>
-#include <wx/notebook.h>
-#include <wx/toolbook.h>
+#include <wx/listbook.h>
 #include <wx/treectrl.h>
-#include <wx/collpane.h>
-#include <wx/combo.h>
+#include <wx/infobar.h>
 #include <wx/grid.h>
 #include <wx/palette.h>
 #include <wx/dirctrl.h>
-#include <wx/choicebk.h>
 #include <wx/artprov.h>
 #include <wx/animate.h>
 #include <wx/dnd.h>
 #include <wx/xml/xml.h>
-
-#ifdef __WXDEBUG__
-#include <wx/debug.h>
-#endif
-
+#include <wx/fswatcher.h>
 #include <queue>
 #include <vector>
 #include <list>
 
 
-
+#ifdef __WXDEBUG__
+#include <wx/debug.h>
+#endif
 
 class MuniPreferences: public wxPropertySheetDialog
 {
@@ -259,6 +240,7 @@ private:
 };
 
 
+/*
 class MuniLog: public wxLogWindow
 {
 public:
@@ -274,49 +256,7 @@ private:
   bool visible;
 
 };
-
-class MuniAbout
-{
-public:
-  MuniAbout(const wxIcon&);
-};
-
-
-class MuniHelp: public wxFrame
-{
-public:
-  MuniHelp(MuniConfig *);
-  virtual ~MuniHelp();
-
-private:
-
-  wxHtmlWindow *web;
-  wxString home;
-  wxTextCtrl *turl;
-  MuniConfig *config;
-  wxToolBar *tbar;
-  wxMenu *menuGo,*menuView;
-
-  void OnClose(wxCloseEvent&);
-  void OnSize(wxSizeEvent&);
-  void FileClose(wxCommandEvent& WXUNUSED(event));
-  void FileOpen(wxCommandEvent& WXUNUSED(event));
-  void GoBack(wxCommandEvent& WXUNUSED(event));
-  void GoForward(wxCommandEvent& WXUNUSED(event));
-  void GoHome(wxCommandEvent& WXUNUSED(event));
-  void ViewStop(wxCommandEvent& WXUNUSED(event));
-  void ViewReload(wxCommandEvent& WXUNUSED(event));
-  void GoUrl(wxCommandEvent& WXUNUSED(event));
-  void GetUrl(wxCommandEvent&);
-  void SelUrl(wxCommandEvent&);
-  void PrintHref(wxHtmlCellEvent&);
-  void ClickedUrl(wxHtmlLinkEvent&);
-  void DefaultBrowser(wxCommandEvent& WXUNUSED(event));
-  void UpdateControls();
-  void HelpAbout(wxCommandEvent& WXUNUSED(event));
-
-  DECLARE_EVENT_TABLE()
-};
+*/
 
 
 // class MuniGraph: public wxPanel
@@ -375,63 +315,62 @@ private:
 
 class MuniThumbCanvas: public wxWindow
 {
+  wxBitmap icon;
+  void OnPaint(wxPaintEvent& WXUNUSED(event));
+
 public:
   MuniThumbCanvas(wxWindow *, const wxBitmap&);
-  //  virtual ~MuniThumbCanvas() {}
   void SetIcon(const wxBitmap&);
   void SetIcon(const wxImage&);
-
-private:
-
-  wxBitmap icon;
-
-  void OnPaint(wxPaintEvent& WXUNUSED(event));
-  void DrawIcon();
-
-  DECLARE_EVENT_TABLE()
 };
 
 
 
-class MuniBaseDialog: public wxDialog
-{
-public:
-  MuniBaseDialog(wxWindow *,const wxString&,MuniConfig *,const wxString&);
-  virtual ~MuniBaseDialog();
+// class MuniDialog: public wxDialog
+// {
+//   MuniConfig *config;
+//   MuniProcess *mproc;
+//   wxTimer timer;
+//   wxAnimationCtrl *anim;
+//   wxButton *execute, *stop;
+//   wxStaticText *status;
+//   wxString action, help;
+//   long index;
+//   wxArrayString out;
 
-protected:
+//   void OnHelp(wxCommandEvent&);
+//   void OnExecute(wxCommandEvent&);
+//   void OnStop(wxCommandEvent&);
+//   void OnTimer(wxTimerEvent&);
+//   void OnFinish(wxProcessEvent&);
+//   void OnStdButton(wxCommandEvent&);
 
-  MuniConfig *config;
-  MuniPipe pipe;
-  wxTimer timer;
-  bool running, readonly;
-  int exitcode;
-  wxString file,workingfile;
+//  protected:
 
-  void OnTimer(wxTimerEvent&);
-  void OnFinish(wxProcessEvent&);
-  bool RunProcess(MuniProcess *p);
+//   wxString fitsname, tmpfits;
+
+//   void SetAction(const wxString&);
+//   void SetHelp(const wxString&);
+//   void SetStatus(const wxString&);
+//   wxString Parser(const wxString&, const wxString&) const;
+//   //  void Write(const wxString&, ...);
+//   wxSizer *CreateStatusSizer(const wxString&);
+//   wxSizer *CreateButtonSizer(long);
+//   wxArrayString GetOutput() const;
+//   wxArrayString GetLastOutput();
+//   virtual void OnInput(MuniProcess *) = 0;
+//   virtual void OnOutput(const wxArrayString&);
+//   virtual void CleanDraw() const {}
+//   //  virtual void DrawStars(const wxArrayString&) const;
+
+//   // public:
+//   MuniDialog(wxWindow *, MuniConfig *, const wxString&, const wxString&);
+//   virtual ~MuniDialog();
+
+// };
 
 
-};
-
-class MuniPhotometryLayer
-{
-public:
- MuniPhotometryLayer(): gold(wxColour(255,215,0,240)),DarkOrange2(wxColour(238,118,0)),hwhm(1) {}
-  MuniLayer GetLayer() const;
-  bool IsEmpty() const { return objects.empty(); }
-  void DrawObjects(const std::vector<double>&,const std::vector<double>&,const std::vector<double>&);
-  void SetHWHM(double x) { hwhm = x; }
-
-private:
-
-  const wxColour gold,DarkOrange2;
-  std::vector<wxObject *> objects;
-  double hwhm;
-
-};
-
+/*
 class MuniPhotometry: public MuniBaseDialog
 {
 public:
@@ -466,38 +405,93 @@ private:
   void OnPhotometry(wxCommandEvent&);
 
 };
+*/
+
+class MuniFind: public wxDialog
+{
+  MuniConfig *config;
+  MuniProcess *mproc;
+  wxTimer timer;
+  wxAnimationCtrl *anim;
+  wxWindow *plot;
+  wxString fitsname, tmpfits;
+  const FitsArray array;
+  double fwhm, thresh, satur;
+  long index;
+  wxButton *start, *stop;
+  wxStaticText *status;
+
+  double InitSatur() const;
+  wxString Parser(const wxString&) const;
+  bool StarParser(const wxString&, long *, double *, double *) const;
+  void DrawStars(long, long, const wxArrayString&) const;
+  void CleanDraw() const;
+  long LastStar(const wxString&) const;
+
+  void OnHelp(wxCommandEvent&);
+  void OnFind(wxCommandEvent&);
+  void OnStop(wxCommandEvent&);
+  void OnTimer(wxTimerEvent&);
+  void OnFindFinish(wxProcessEvent&);
+
+  void OnFwhm(wxSpinDoubleEvent&);
+  void OnThresh(wxSpinDoubleEvent&);
+  void OnSatur(wxSpinDoubleEvent&);
+  void OnStdButton(wxCommandEvent&);
+
+ public:
+  MuniFind(wxWindow *, MuniConfig *, const wxString&, const FitsHdu&);
+  virtual ~MuniFind();
+
+  void SetPoint(int,int) const;
+
+};
 
 
 class MuniCone: public wxDialog
 {
 public:
-  MuniCone(wxWindow *,MuniConfig *);
+  MuniCone(wxWindow *,MuniConfig *,const wxString& =wxEmptyString);
   virtual ~MuniCone();
   wxString GetPath() const { return tmpfile; }
 
 private:
 
   MuniConfig *config;
+  MuniProcess *mproc;
+  wxTimer timer;
   VOCatConf catconf;
   wxAnimationCtrl *throbber;
-  wxStaticText *status;
-  wxButton *search;
-  bool running, apply;
-  MuniPipe pipe;
+  wxStaticText *status, *johnson_label;
+  wxTextCtrl *objentry, *alpha, *delta;
+  wxButton *search, *stop;
+  wxCheckBox *johnson;
+  bool apply;
 
-  wxSpinCtrlDouble *wradius, *walpha, *wdelta;
-  wxString tmpfile;
+  wxSpinCtrlDouble *radius, *magmin, *magmax;
+  wxString tmpfile, catfits, object;
+  double ra,dec;
+  long index;
   std::list<int> ids;
 
   void OnUpdateUI(wxUpdateUIEvent&);
   void OnOk(wxCommandEvent&);
-  void OnCancel(wxCommandEvent&);
   void OnClose(wxCloseEvent&);
+  void OnHelp(wxCommandEvent&);
+  void OnObjectName(wxCommandEvent&);
+  void OnObjectEnter(wxCommandEvent&);
+  void OnGetCoo(wxCommandEvent&);
+  void OnRightAscension(wxCommandEvent&);
+  void OnDeclination(wxCommandEvent&);
   void OnSearch(wxCommandEvent&);
   void OnFinish(wxProcessEvent&);
-  //  void OnSave(wxFileDirPickerEvent&);
+  void OnStop(wxCommandEvent&);
+  void OnTimer(wxTimerEvent&);
   void OnService(wxCommandEvent&);
-  bool ParseOutput(const wxArrayString&);
+  wxString ParseOutput(const wxArrayString&);
+  void Resolve();
+  void ResolveFinish(wxProcessEvent&);
+  double deg(const wxString&, double =1.0);
 
 };
 
@@ -731,7 +725,6 @@ private:
 
 };
 
-
 class MuniColoring: public wxDialog
 {
 public:
@@ -773,69 +766,56 @@ private:
 };
 
 
-// assumed to by moved to view related (self-contained) header file
-class MuniResume: public wxHtmlWindow
-{
-public:
-
-  MuniResume(wxWindow *, const MuniConfig *, const FitsMeta&);
-  void CreatePage(const FitsMeta&);
-  virtual ~MuniResume();
-
-private:
-
-  const MuniConfig *config;
-  const wxString filename;
-  std::list<wxString> files;
-
-  void OnLinkClicked(wxHtmlLinkEvent&);
-
-};
-
-
 class MuniDisplay: public wxWindow
 {
 public:
   MuniDisplay(wxWindow *w, MuniConfig *);
 
-  bool SetHdu(const FitsHdu&, const wxImage&, bool);
+  bool SetHdu(const FitsHdu&, const wxImage&);
+  void SetInitShrink(int);
   //  virtual void SetOverlay(wxInputStream&);
   virtual void SetStars(FitsTable&);
+  void InvokeRendering();
   void StopRendering();
-  int GetFitShrink() const;
-  FitsZoom GetZoom() const;
   FitsTone GetTone() const;
   FitsItt GetItt() const;
   FitsPalette GetPalette() const;
   FitsColor GetColor() const;
   wxImage GetImage();
   void ShowPanel(bool);
-  void ShowTune(bool =true);
+  void ShowCaption(bool);
+  void ShowMagnifier(bool);
+  void ShowTune(bool=true);
   void ShowGrid(bool =true);
   void ShowSources(bool, const FitsFile&);
+  void OnFind(const wxString&, const FitsHdu&);
   void Photometry(const wxString&);
   void Astrometry(const wxString&,const FitsTable&);
   void Calibrate(const wxString&);
+  bool IsTuneShown() const;
+  bool IsMagnifierShown() const;
+  wxSize GetCanvasSize() const;
 
 private:
 
   MuniConfig *config;
   FitsArray array;
   wxImage thumb;
-  wxPanel *panel;
+  wxPanel *panel, *caption;
   wxWindow *canvas;
-  FitsHisto hist;
-  wxFrame *tune;
+  wxFrame *tune, *magnifier;
+  MuniFind *find;
   MuniAstrometry *astrometry;
-  //  MuniPhotometry *photometry;
 
   void OnClose(wxCloseEvent&);
   void OnCloseTune(wxCloseEvent&);
   void OnCloseAstrometry(wxCommandEvent&);
-  //  void OnClosePhotometry(wxCloseEvent&);
+  void OnCloseFind(wxCommandEvent&);
   void OnLeaveFullscreen(wxCommandEvent&);
-  void OnEnterWin(wxMouseEvent&);
-  void OnLeaveWin(wxMouseEvent&);
+  void OnRenderFinish(MuniRenderEvent&);
+  void OnClick(MuniClickEvent&);
+  void OnMouseMotion(MuniSlewEvent&);
+  void OnTuneFine(MuniTuneEvent&);
 
 };
 
@@ -908,16 +888,57 @@ public:
 };
 
 
+class MuniExtensionList: public wxListView
+{
+  void OnExtChanged(wxListEvent&);
+  wxBitmap DrawIcon(int, int);
+
+public:
+  MuniExtensionList(wxWindow*);
+  void Set(const FitsMeta&);
+  void ChangeSelection(int);
+
+};
+
+class MuniViewZoom: public wxChoice
+{
+  static const int nratios, oratio;
+  static const int ratios[];
+
+  size_t current, bestfit;
+  wxArrayString zoomlist;
+  wxString Label(int) const;
+  void SelectItem(int);
+  void OnChoiceSelect(wxCommandEvent&);
+
+public:
+  MuniViewZoom(wxWindow *, wxWindowID, int);
+
+  int GetValue() const;
+  void SelectIn();
+  void SelectOut();
+  void SelectNormalSize();
+  void SelectBestFitSize();
+  void SetBestFit(int,int,int,int);
+  bool IsNormalSize() const;
+  bool IsBestFitSize() const;
+  bool IsIncable() const;
+  bool IsOutable() const;
+
+};
+
 
 class MuniView: public wxFrame
 {
 public:
   MuniView(wxWindow *,MuniConfig *);
+  virtual ~MuniView();
   void LoadFile(const wxString&);
   void SaveFile(const wxString&);
   void LoadMeta(const FitsMeta&);
   void LoadFileBackup(const wxString&, const wxString&);
   bool IsModified() const;
+  void CreateFSWatch();
 
 private:
 
@@ -925,100 +946,105 @@ private:
   friend class FitsExport;
 
   MuniConfig *config;
+  wxFileSystemWatcher *fswatch;
+  bool loadfile;  // trigger for LoadFile during idle
   FitsFile fits;
   FitsMeta meta;
-  int hdusel;
-  bool palinv, palauto;
+  size_t hdusel;
   std::vector<long> viewid;
   wxArrayString backup;
 
   wxThread *loader;
   wxCriticalSection loaderCS;
 
-  wxMenu *menuFile,*menuView,*menuWin;
-  wxComboCtrl* structtree;
-  wxComboBox *zoomctrl;
-  wxWindow *place;
-  MuniColoring *coloring;
-  wxLogWindow *console;
+  wxMenu *menuFile,*menuView,*menuExt;
+  MuniViewZoom *zoomctrl;
+  MuniExtensionList *extlist;
+  MuniSplashing *splash;
+  std::vector<wxWindow *> places;
+  std::vector<int> zoomkeeper;
+  //  MuniColoring *coloring;
+  //  wxLogWindow *console;
+  MuniAphot *aphot;
 
-  void Clear();
 
-  void ReplacePlace(wxWindow *);
-
-  void CreateSplashing();
-  void CreateResume();
-  void CreateHead();
-  void CreateGrid();
-  void CreateDisplay();
-  void DeleteSplashing();
-  void DeleteResume();
-  void DeleteHead();
-  void DeleteGrid();
-  void DeleteDisplay();
-
-  void SetMeta(const FitsMeta&);
+  void SetupPlaces();
+  int GetHduType() const;
+  void InitImage();
+  void InitTable();
+  void InitHead();
+  void UpdatePlace(int,bool);
+  void ResetPlace();
+  int GetHduSel() const;
+  void SetupExtension();
+  void UpdateZoom(const wxSize&);
+  void SendScaleEvent();
   void StopLoading();
+  void SetTitle();
+  MuniDisplay *GetDisplay() const;
 
-  void HduSelect(int);
-  //  void EstimHistos(const FitsFile&);
-  int EstimHduSel(const FitsMeta&);
-  void MenuSelect(int);
-  void TreeSelect(int);
-
-  void OnClose(wxCloseEvent& WXUNUSED(event));
-  void OnSize(wxSizeEvent& WXUNUSED(event));
+  void OnClose(wxCloseEvent& event);
+  void OnIdle(wxIdleEvent& event);
+  void OnSize(wxSizeEvent& event);
+  void OnClick(MuniClickEvent&);
   void FileClose(wxCommandEvent& WXUNUSED(event));
   void FileOpen(wxCommandEvent& WXUNUSED(event));
   //  void NewBrowser(wxCommandEvent& WXUNUSED(event));
   void NewView(wxCommandEvent& WXUNUSED(event));
+  void OnConeSearch(wxCommandEvent&);
+  void OnCloseCone(wxCommandEvent&);
   void OnFullScreen(wxCommandEvent& WXUNUSED(event));
+  void OnMagnifier(wxCommandEvent& WXUNUSED(event));
   void OnTune(wxCommandEvent& WXUNUSED(event));
-  void OnPhotometry(wxCommandEvent& WXUNUSED(event));
+  void OnFind(wxCommandEvent& WXUNUSED(event));
+  void OnAphot(wxCommandEvent& WXUNUSED(event));
+  void OnAphotFinish(wxCommandEvent&);
   void OnAstrometry(wxCommandEvent& WXUNUSED(event));
-  void OnCalibrate(wxCommandEvent& WXUNUSED(event));
   void OnShowGrid(wxCommandEvent&);
   void OnShowSources(wxCommandEvent&);
   void OnHeader(wxCommandEvent& WXUNUSED(event));
-  void HelpGuide(wxCommandEvent& WXUNUSED(event));
   void HelpAbout(wxCommandEvent& WXUNUSED(event));
-  //  void HelpBug(wxCommandEvent& WXUNUSED(event));
-  void ViewLog(wxCommandEvent&);
+  //  void ViewLog(wxCommandEvent&);
   void FileProperties(wxCommandEvent& WXUNUSED(event));
   void OnPreferences(wxCommandEvent& WXUNUSED(event));
   void ShowToolbar(wxCommandEvent&);
-  void OnMenuPalauto(wxCommandEvent&);
-  void OnMenuInverse(wxCommandEvent&);
-  void OnMenuPal(wxCommandEvent&);
-  void OnMenuStructure(wxCommandEvent&);
-  void OnSelectFrame(wxCommandEvent&);
+  void ShowExtlist(wxCommandEvent&);
+  void ShowCaption(wxCommandEvent&);
+  void OnMenuExt(wxCommandEvent&);
   void OnMenuZoom(wxCommandEvent&);
   void OnToolZoom(wxCommandEvent&);
-  void OnCycleZoom(wxCommandEvent&);
-  void OnCyclePal(wxCommandEvent&);
   void OnDetailPanel(wxCommandEvent&);
-  void OnEnterWin(wxMouseEvent&);
   //  void SaveAsText(wxString);
   void FileSave(wxCommandEvent& WXUNUSED(event));
   void FileExport(wxCommandEvent& WXUNUSED(event));
   void OnLoadFinish(FitsOpenEvent&);
-  void SelectPal(const FitsPalette&);
-  void SelectItt(const FitsItt&);
-  void OnTuneFine(MuniTuneEvent&);
-
+  void OnExtChanged(wxListEvent&);
+  void OnUpdateBackward(wxUpdateUIEvent&);
+  void OnUpdateForward(wxUpdateUIEvent&);
+  void OnUpdateShowGrid(wxUpdateUIEvent&);
+  void OnUpdateShowSources(wxUpdateUIEvent&);
+  void OnUpdateShowTune(wxUpdateUIEvent&);
+  void OnUpdateShowMagnifier(wxUpdateUIEvent&);
+  void OnUpdateZoomFit(wxUpdateUIEvent&);
+  void OnUpdateZoom100(wxUpdateUIEvent&);
+  void OnUpdateZoomIn(wxUpdateUIEvent&);
+  void OnUpdateZoomOut(wxUpdateUIEvent&);
+  void OnUpdateFullScreen(wxUpdateUIEvent&);
+  /*
   void Coloring(wxCommandEvent& WXUNUSED(event));
   void OnColoringFinish(wxCommandEvent&);
-
-  /*
-  void OnPhotometry(MuniDialogEvent&);
-  void OnPhotometryFinish(wxProcessEvent&);
   */
 
+  void OnDraw(MuniDrawEvent&);
+
+  // File monitor
+  void OnFileSystemEvent(wxFileSystemWatcherEvent&);
+  void MonitorFile(const wxString&);
+  double MonitorDelay(const wxString&) const;
+
   void LoadStart(const wxString&);
   void ExportStart(const wxString&);
-  void OnUpdateInfos(wxUpdateUIEvent&);
 
-  void OnResumeHdu(MuniHduEvent&);
   int Unsaved(const wxString&);
   void RemoveBackup();
 
@@ -1490,8 +1516,6 @@ private:
 
   MuniConfig *config;
   MuniArchive *archive;
-  //  MuniConsole *console;
-  wxLogWindow *console;
   wxMenu *menuFile, *menuView, *menuAct, *menuArrange, *menuLabels, *menuHelp;
   MuniListWindow *list;
   MuniView *view;
@@ -1526,8 +1550,6 @@ private:
   void NewBrowser(wxCommandEvent& WXUNUSED(event));
   void NewView(wxCommandEvent& WXUNUSED(event));
   void OnView(wxCommandEvent& WXUNUSED(event));
-  void ViewLog(wxCommandEvent&);
-  void ShowLog(wxCommandEvent&);
   void OnActivated(wxListEvent&);
   void OnShowToolbar(wxCommandEvent&);
   void OnShowCollector(wxCommandEvent&);
@@ -1541,7 +1563,7 @@ private:
   void OnAverage(wxCommandEvent& WXUNUSED(event));
   void OnDarkbat(wxCommandEvent& WXUNUSED(event));
   void OnList(wxCommandEvent&);
-  void HelpHelp(wxCommandEvent& WXUNUSED(event));
+  //  void HelpHelp(wxCommandEvent& WXUNUSED(event));
   void HelpAbout(wxCommandEvent& WXUNUSED(event));
   //  void HelpBug(wxCommandEvent& WXUNUSED(event));
   void OnNewArchive(wxCommandEvent&);
@@ -1579,7 +1601,12 @@ class XMunipack: public wxApp
   bool OnInit();
   int OnExit();
   void OnFatalException();
+  void OnEventLoopEnter(wxEventLoopBase*);
 
   MuniConfig *config;
+  MuniView *view;
+  MuniBrowser *browser;
 
 };
+
+#endif
